Package: distributional
Title: Vectorised Probability Distributions
Version: 0.7.0
Authors@R: 
    c(person(given = "Mitchell",
             family = "O'Hara-Wild",
             role = c("aut", "cre"),
             email = "mail@mitchelloharawild.com",
             comment = c(ORCID = "0000-0001-6729-7695")),
      person(given = "Matthew",
             family = "Kay",
             role = c("aut"),
             comment = c(ORCID = "0000-0001-9446-0419")),
      person(given = "Alex",
             family = "Hayes",
             role = c("aut"),
             comment = c(ORCID = "0000-0002-4985-5160")),
      person(given = "Rob",
             family = "Hyndman",
             role = c("aut"),
             comment = c(ORCID = "0000-0002-2140-5352")),
      person(given = "Earo",
             family = "Wang",
             role = c("ctb"),
             comment = c(ORCID = "0000-0001-6448-5260")),
      person(given = "Vencislav",
             family = "Popov",
             role = c("ctb"),
             comment = c(ORCID = "0000-0002-8073-4199")))
Description: Vectorised distribution objects with tools for manipulating, 
    visualising, and using probability distributions. Designed to allow model
    prediction outputs to return distributions rather than their parameters, 
    allowing users to directly interact with predictive distributions in a
    data-oriented workflow. In addition to providing generic replacements for
    p/d/q/r functions, other useful statistics can be computed including means,
    variances, intervals, and highest density regions.
License: GPL-3
Depends: R (>= 4.0.0)
Imports: vctrs (>= 0.3.0), rlang (>= 0.4.5), generics, stats, numDeriv,
        utils, lifecycle, pillar
Suggests: testthat (>= 2.1.0), covr, mvtnorm, actuar (>= 2.0.0), evd,
        ggdist, ggplot2, gk, LaplacesDemon, pkgdown
RdMacros: lifecycle
URL: https://pkg.mitchelloharawild.com/distributional/,
        https://github.com/mitchelloharawild/distributional
BugReports: https://github.com/mitchelloharawild/distributional/issues
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-17 07:17:03 UTC; mitchell
Author: Mitchell O'Hara-Wild [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6729-7695>),
  Matthew Kay [aut] (ORCID: <https://orcid.org/0000-0001-9446-0419>),
  Alex Hayes [aut] (ORCID: <https://orcid.org/0000-0002-4985-5160>),
  Rob Hyndman [aut] (ORCID: <https://orcid.org/0000-0002-2140-5352>),
  Earo Wang [ctb] (ORCID: <https://orcid.org/0000-0001-6448-5260>),
  Vencislav Popov [ctb] (ORCID: <https://orcid.org/0000-0002-8073-4199>)
Maintainer: Mitchell O'Hara-Wild <mail@mitchelloharawild.com>
Repository: CRAN
Date/Publication: 2026-03-17 08:30:44 UTC
Built: R 4.6.0; ; 2026-04-26 02:23:16 UTC; unix
