# Generated by roxygen2: do not edit by hand

S3method("$",hilo)
S3method("[[",distribution)
S3method("dimnames<-",distribution)
S3method("names<-",hilo)
S3method(.DollarNames,hilo)
S3method(Math,dist_default)
S3method(Math,dist_lognormal)
S3method(Math,dist_na)
S3method(Math,dist_normal)
S3method(Math,dist_sample)
S3method(Math,dist_transformed)
S3method(Ops,dist_default)
S3method(Ops,dist_na)
S3method(Ops,dist_normal)
S3method(Ops,dist_sample)
S3method(Ops,dist_transformed)
S3method(cdf,dist_bernoulli)
S3method(cdf,dist_beta)
S3method(cdf,dist_binomial)
S3method(cdf,dist_burr)
S3method(cdf,dist_categorical)
S3method(cdf,dist_cauchy)
S3method(cdf,dist_chisq)
S3method(cdf,dist_default)
S3method(cdf,dist_degenerate)
S3method(cdf,dist_exponential)
S3method(cdf,dist_f)
S3method(cdf,dist_gamma)
S3method(cdf,dist_geometric)
S3method(cdf,dist_gh)
S3method(cdf,dist_gk)
S3method(cdf,dist_gumbel)
S3method(cdf,dist_hypergeometric)
S3method(cdf,dist_inflated)
S3method(cdf,dist_inverse_exponential)
S3method(cdf,dist_inverse_gamma)
S3method(cdf,dist_inverse_gaussian)
S3method(cdf,dist_laplace)
S3method(cdf,dist_logarithmic)
S3method(cdf,dist_logistic)
S3method(cdf,dist_lognormal)
S3method(cdf,dist_mixture)
S3method(cdf,dist_multinomial)
S3method(cdf,dist_mvnorm)
S3method(cdf,dist_mvt)
S3method(cdf,dist_na)
S3method(cdf,dist_negbin)
S3method(cdf,dist_normal)
S3method(cdf,dist_pareto)
S3method(cdf,dist_percentile)
S3method(cdf,dist_poisson)
S3method(cdf,dist_poisson_inverse_gaussian)
S3method(cdf,dist_sample)
S3method(cdf,dist_student_t)
S3method(cdf,dist_studentized_range)
S3method(cdf,dist_transformed)
S3method(cdf,dist_truncated)
S3method(cdf,dist_uniform)
S3method(cdf,dist_weibull)
S3method(cdf,dist_wrap)
S3method(cdf,distribution)
S3method(covariance,default)
S3method(covariance,dist_bernoulli)
S3method(covariance,dist_beta)
S3method(covariance,dist_binomial)
S3method(covariance,dist_burr)
S3method(covariance,dist_categorical)
S3method(covariance,dist_cauchy)
S3method(covariance,dist_chisq)
S3method(covariance,dist_default)
S3method(covariance,dist_degenerate)
S3method(covariance,dist_dirichlet)
S3method(covariance,dist_exponential)
S3method(covariance,dist_f)
S3method(covariance,dist_gamma)
S3method(covariance,dist_geometric)
S3method(covariance,dist_gumbel)
S3method(covariance,dist_hypergeometric)
S3method(covariance,dist_inflated)
S3method(covariance,dist_inverse_exponential)
S3method(covariance,dist_inverse_gamma)
S3method(covariance,dist_inverse_gaussian)
S3method(covariance,dist_logarithmic)
S3method(covariance,dist_logistic)
S3method(covariance,dist_lognormal)
S3method(covariance,dist_mixture)
S3method(covariance,dist_multinomial)
S3method(covariance,dist_mvnorm)
S3method(covariance,dist_mvt)
S3method(covariance,dist_na)
S3method(covariance,dist_negbin)
S3method(covariance,dist_normal)
S3method(covariance,dist_pareto)
S3method(covariance,dist_poisson)
S3method(covariance,dist_poisson_inverse_gaussian)
S3method(covariance,dist_sample)
S3method(covariance,dist_student_t)
S3method(covariance,dist_transformed)
S3method(covariance,dist_uniform)
S3method(covariance,dist_weibull)
S3method(covariance,distribution)
S3method(covariance,numeric)
S3method(density,dist_bernoulli)
S3method(density,dist_beta)
S3method(density,dist_binomial)
S3method(density,dist_burr)
S3method(density,dist_categorical)
S3method(density,dist_cauchy)
S3method(density,dist_chisq)
S3method(density,dist_default)
S3method(density,dist_degenerate)
S3method(density,dist_dirichlet)
S3method(density,dist_exponential)
S3method(density,dist_f)
S3method(density,dist_gamma)
S3method(density,dist_geometric)
S3method(density,dist_gh)
S3method(density,dist_gk)
S3method(density,dist_gumbel)
S3method(density,dist_horseshoe)
S3method(density,dist_hypergeometric)
S3method(density,dist_inflated)
S3method(density,dist_inverse_exponential)
S3method(density,dist_inverse_gamma)
S3method(density,dist_inverse_gaussian)
S3method(density,dist_laplace)
S3method(density,dist_logarithmic)
S3method(density,dist_logistic)
S3method(density,dist_lognormal)
S3method(density,dist_mixture)
S3method(density,dist_multinomial)
S3method(density,dist_mvnorm)
S3method(density,dist_mvt)
S3method(density,dist_na)
S3method(density,dist_negbin)
S3method(density,dist_normal)
S3method(density,dist_pareto)
S3method(density,dist_percentile)
S3method(density,dist_poisson)
S3method(density,dist_poisson_inverse_gaussian)
S3method(density,dist_sample)
S3method(density,dist_student_t)
S3method(density,dist_transformed)
S3method(density,dist_truncated)
S3method(density,dist_uniform)
S3method(density,dist_weibull)
S3method(density,dist_wrap)
S3method(density,distribution)
S3method(dim,dist_default)
S3method(dim,dist_dirichlet)
S3method(dim,dist_mixture)
S3method(dim,dist_multinomial)
S3method(dim,dist_mvnorm)
S3method(dim,dist_mvt)
S3method(dimnames,distribution)
S3method(distributional::cdf,dist_gev)
S3method(distributional::cdf,dist_gpd)
S3method(distributional::covariance,dist_gev)
S3method(distributional::covariance,dist_gpd)
S3method(distributional::generate,dist_gev)
S3method(distributional::generate,dist_gpd)
S3method(distributional::log_density,dist_gev)
S3method(distributional::log_density,dist_gpd)
S3method(family,dist_default)
S3method(family,distribution)
S3method(format,dist_bernoulli)
S3method(format,dist_beta)
S3method(format,dist_binomial)
S3method(format,dist_burr)
S3method(format,dist_categorical)
S3method(format,dist_cauchy)
S3method(format,dist_chisq)
S3method(format,dist_default)
S3method(format,dist_degenerate)
S3method(format,dist_dirichlet)
S3method(format,dist_exponential)
S3method(format,dist_f)
S3method(format,dist_gamma)
S3method(format,dist_geometric)
S3method(format,dist_gev)
S3method(format,dist_gh)
S3method(format,dist_gk)
S3method(format,dist_gpd)
S3method(format,dist_gumbel)
S3method(format,dist_horseshoe)
S3method(format,dist_hypergeometric)
S3method(format,dist_inflated)
S3method(format,dist_inverse_exponential)
S3method(format,dist_inverse_gamma)
S3method(format,dist_inverse_gaussian)
S3method(format,dist_laplace)
S3method(format,dist_logarithmic)
S3method(format,dist_logistic)
S3method(format,dist_lognormal)
S3method(format,dist_mixture)
S3method(format,dist_multinomial)
S3method(format,dist_mvnorm)
S3method(format,dist_mvt)
S3method(format,dist_na)
S3method(format,dist_negbin)
S3method(format,dist_normal)
S3method(format,dist_pareto)
S3method(format,dist_percentile)
S3method(format,dist_poisson)
S3method(format,dist_poisson_inverse_gaussian)
S3method(format,dist_sample)
S3method(format,dist_student_t)
S3method(format,dist_studentized_range)
S3method(format,dist_transformed)
S3method(format,dist_truncated)
S3method(format,dist_uniform)
S3method(format,dist_weibull)
S3method(format,dist_wrap)
S3method(format,distribution)
S3method(format,hdr)
S3method(format,hilo)
S3method(format,pillar_distribution)
S3method(format,support_region)
S3method(generate,dist_bernoulli)
S3method(generate,dist_beta)
S3method(generate,dist_binomial)
S3method(generate,dist_burr)
S3method(generate,dist_categorical)
S3method(generate,dist_cauchy)
S3method(generate,dist_chisq)
S3method(generate,dist_default)
S3method(generate,dist_degenerate)
S3method(generate,dist_dirichlet)
S3method(generate,dist_exponential)
S3method(generate,dist_f)
S3method(generate,dist_gamma)
S3method(generate,dist_geometric)
S3method(generate,dist_gh)
S3method(generate,dist_gk)
S3method(generate,dist_gumbel)
S3method(generate,dist_horseshoe)
S3method(generate,dist_hypergeometric)
S3method(generate,dist_inflated)
S3method(generate,dist_inverse_exponential)
S3method(generate,dist_inverse_gamma)
S3method(generate,dist_inverse_gaussian)
S3method(generate,dist_laplace)
S3method(generate,dist_logarithmic)
S3method(generate,dist_logistic)
S3method(generate,dist_lognormal)
S3method(generate,dist_mixture)
S3method(generate,dist_multinomial)
S3method(generate,dist_mvnorm)
S3method(generate,dist_mvt)
S3method(generate,dist_na)
S3method(generate,dist_negbin)
S3method(generate,dist_normal)
S3method(generate,dist_pareto)
S3method(generate,dist_percentile)
S3method(generate,dist_poisson)
S3method(generate,dist_poisson_inverse_gaussian)
S3method(generate,dist_sample)
S3method(generate,dist_student_t)
S3method(generate,dist_transformed)
S3method(generate,dist_uniform)
S3method(generate,dist_weibull)
S3method(generate,dist_wrap)
S3method(generate,distribution)
S3method(has_symmetry,dist_beta)
S3method(has_symmetry,dist_binomial)
S3method(has_symmetry,dist_cauchy)
S3method(has_symmetry,dist_default)
S3method(has_symmetry,dist_laplace)
S3method(has_symmetry,dist_logistic)
S3method(has_symmetry,dist_normal)
S3method(has_symmetry,dist_student_t)
S3method(has_symmetry,dist_uniform)
S3method(has_symmetry,distribution)
S3method(hdr,default)
S3method(hdr,dist_default)
S3method(hdr,distribution)
S3method(hilo,default)
S3method(hilo,dist_default)
S3method(hilo,distribution)
S3method(is.na,hilo)
S3method(kurtosis,dist_bernoulli)
S3method(kurtosis,dist_beta)
S3method(kurtosis,dist_binomial)
S3method(kurtosis,dist_categorical)
S3method(kurtosis,dist_cauchy)
S3method(kurtosis,dist_chisq)
S3method(kurtosis,dist_degenerate)
S3method(kurtosis,dist_exponential)
S3method(kurtosis,dist_f)
S3method(kurtosis,dist_gamma)
S3method(kurtosis,dist_geometric)
S3method(kurtosis,dist_gumbel)
S3method(kurtosis,dist_hypergeometric)
S3method(kurtosis,dist_laplace)
S3method(kurtosis,dist_logistic)
S3method(kurtosis,dist_lognormal)
S3method(kurtosis,dist_na)
S3method(kurtosis,dist_negbin)
S3method(kurtosis,dist_normal)
S3method(kurtosis,dist_poisson)
S3method(kurtosis,dist_uniform)
S3method(kurtosis,dist_weibull)
S3method(kurtosis,distribution)
S3method(likelihood,dist_default)
S3method(likelihood,distribution)
S3method(log_cdf,dist_default)
S3method(log_cdf,dist_lognormal)
S3method(log_cdf,dist_na)
S3method(log_cdf,dist_normal)
S3method(log_cdf,distribution)
S3method(log_density,dist_bernoulli)
S3method(log_density,dist_beta)
S3method(log_density,dist_binomial)
S3method(log_density,dist_burr)
S3method(log_density,dist_cauchy)
S3method(log_density,dist_chisq)
S3method(log_density,dist_default)
S3method(log_density,dist_dirichlet)
S3method(log_density,dist_exponential)
S3method(log_density,dist_f)
S3method(log_density,dist_gamma)
S3method(log_density,dist_geometric)
S3method(log_density,dist_gh)
S3method(log_density,dist_gk)
S3method(log_density,dist_gumbel)
S3method(log_density,dist_horseshoe)
S3method(log_density,dist_hypergeometric)
S3method(log_density,dist_inverse_exponential)
S3method(log_density,dist_inverse_gamma)
S3method(log_density,dist_inverse_gaussian)
S3method(log_density,dist_laplace)
S3method(log_density,dist_logarithmic)
S3method(log_density,dist_logistic)
S3method(log_density,dist_lognormal)
S3method(log_density,dist_multinomial)
S3method(log_density,dist_mvnorm)
S3method(log_density,dist_mvt)
S3method(log_density,dist_na)
S3method(log_density,dist_negbin)
S3method(log_density,dist_normal)
S3method(log_density,dist_pareto)
S3method(log_density,dist_poisson)
S3method(log_density,dist_poisson_inverse_gaussian)
S3method(log_density,dist_student_t)
S3method(log_density,dist_uniform)
S3method(log_density,dist_weibull)
S3method(log_density,dist_wrap)
S3method(log_density,distribution)
S3method(log_likelihood,dist_default)
S3method(log_likelihood,distribution)
S3method(log_quantile,dist_default)
S3method(log_quantile,dist_lognormal)
S3method(log_quantile,dist_na)
S3method(log_quantile,dist_normal)
S3method(log_quantile,distribution)
S3method(mean,dist_bernoulli)
S3method(mean,dist_beta)
S3method(mean,dist_binomial)
S3method(mean,dist_burr)
S3method(mean,dist_categorical)
S3method(mean,dist_cauchy)
S3method(mean,dist_chisq)
S3method(mean,dist_default)
S3method(mean,dist_degenerate)
S3method(mean,dist_dirichlet)
S3method(mean,dist_exponential)
S3method(mean,dist_f)
S3method(mean,dist_gamma)
S3method(mean,dist_geometric)
S3method(mean,dist_gev)
S3method(mean,dist_gpd)
S3method(mean,dist_gumbel)
S3method(mean,dist_hypergeometric)
S3method(mean,dist_inflated)
S3method(mean,dist_inverse_exponential)
S3method(mean,dist_inverse_gamma)
S3method(mean,dist_inverse_gaussian)
S3method(mean,dist_laplace)
S3method(mean,dist_logarithmic)
S3method(mean,dist_logistic)
S3method(mean,dist_lognormal)
S3method(mean,dist_mixture)
S3method(mean,dist_multinomial)
S3method(mean,dist_mvnorm)
S3method(mean,dist_mvt)
S3method(mean,dist_na)
S3method(mean,dist_negbin)
S3method(mean,dist_normal)
S3method(mean,dist_pareto)
S3method(mean,dist_percentile)
S3method(mean,dist_poisson)
S3method(mean,dist_poisson_inverse_gaussian)
S3method(mean,dist_sample)
S3method(mean,dist_student_t)
S3method(mean,dist_transformed)
S3method(mean,dist_truncated)
S3method(mean,dist_uniform)
S3method(mean,dist_weibull)
S3method(mean,distribution)
S3method(median,dist_default)
S3method(median,dist_mvnorm)
S3method(median,dist_sample)
S3method(median,distribution)
S3method(parameters,dist_default)
S3method(parameters,dist_wrap)
S3method(parameters,distribution)
S3method(pillar_shaft,distribution)
S3method(print,dist_default)
S3method(quantile,dist_bernoulli)
S3method(quantile,dist_beta)
S3method(quantile,dist_binomial)
S3method(quantile,dist_burr)
S3method(quantile,dist_categorical)
S3method(quantile,dist_cauchy)
S3method(quantile,dist_chisq)
S3method(quantile,dist_default)
S3method(quantile,dist_degenerate)
S3method(quantile,dist_exponential)
S3method(quantile,dist_f)
S3method(quantile,dist_gamma)
S3method(quantile,dist_geometric)
S3method(quantile,dist_gh)
S3method(quantile,dist_gk)
S3method(quantile,dist_gumbel)
S3method(quantile,dist_hypergeometric)
S3method(quantile,dist_inflated)
S3method(quantile,dist_inverse_exponential)
S3method(quantile,dist_inverse_gamma)
S3method(quantile,dist_inverse_gaussian)
S3method(quantile,dist_laplace)
S3method(quantile,dist_logarithmic)
S3method(quantile,dist_logistic)
S3method(quantile,dist_lognormal)
S3method(quantile,dist_mixture)
S3method(quantile,dist_mvnorm)
S3method(quantile,dist_mvt)
S3method(quantile,dist_na)
S3method(quantile,dist_negbin)
S3method(quantile,dist_normal)
S3method(quantile,dist_pareto)
S3method(quantile,dist_percentile)
S3method(quantile,dist_poisson)
S3method(quantile,dist_poisson_inverse_gaussian)
S3method(quantile,dist_sample)
S3method(quantile,dist_student_t)
S3method(quantile,dist_studentized_range)
S3method(quantile,dist_transformed)
S3method(quantile,dist_truncated)
S3method(quantile,dist_uniform)
S3method(quantile,dist_weibull)
S3method(quantile,dist_wrap)
S3method(quantile,distribution)
S3method(skewness,dist_bernoulli)
S3method(skewness,dist_beta)
S3method(skewness,dist_binomial)
S3method(skewness,dist_categorical)
S3method(skewness,dist_cauchy)
S3method(skewness,dist_chisq)
S3method(skewness,dist_degenerate)
S3method(skewness,dist_exponential)
S3method(skewness,dist_f)
S3method(skewness,dist_gamma)
S3method(skewness,dist_geometric)
S3method(skewness,dist_gumbel)
S3method(skewness,dist_hypergeometric)
S3method(skewness,dist_laplace)
S3method(skewness,dist_logistic)
S3method(skewness,dist_lognormal)
S3method(skewness,dist_na)
S3method(skewness,dist_negbin)
S3method(skewness,dist_normal)
S3method(skewness,dist_poisson)
S3method(skewness,dist_sample)
S3method(skewness,dist_uniform)
S3method(skewness,dist_weibull)
S3method(skewness,distribution)
S3method(stats::density,dist_gev)
S3method(stats::density,dist_gpd)
S3method(stats::median,dist_gev)
S3method(stats::median,dist_gpd)
S3method(stats::quantile,dist_gev)
S3method(stats::quantile,dist_gpd)
S3method(sum,distribution)
S3method(support,dist_categorical)
S3method(support,dist_default)
S3method(support,dist_dirichlet)
S3method(support,dist_gpd)
S3method(support,dist_horseshoe)
S3method(support,dist_percentile)
S3method(support,dist_sample)
S3method(support,dist_transformed)
S3method(support,distribution)
S3method(variance,default)
S3method(variance,dist_default)
S3method(variance,dist_laplace)
S3method(variance,distribution)
S3method(variance,matrix)
S3method(variance,numeric)
S3method(vec_arith,distribution)
S3method(vec_arith,hilo)
S3method(vec_arith.distribution,default)
S3method(vec_arith.numeric,distribution)
S3method(vec_arith.numeric,hilo)
S3method(vec_cast,character.distribution)
S3method(vec_cast,character.hilo)
S3method(vec_cast,distribution.distribution)
S3method(vec_cast,distribution.double)
S3method(vec_cast,distribution.integer)
S3method(vec_math,distribution)
S3method(vec_math,hilo)
S3method(vec_ptype2,distribution.distribution)
S3method(vec_ptype2,distribution.double)
S3method(vec_ptype2,distribution.integer)
S3method(vec_ptype2,double.distribution)
S3method(vec_ptype2,hilo.hilo)
S3method(vec_ptype2,integer.distribution)
S3method(vec_ptype_abbr,distribution)
S3method(vec_ptype_abbr,support_region)
export(cdf)
export(covariance)
export(dist_bernoulli)
export(dist_beta)
export(dist_binomial)
export(dist_burr)
export(dist_categorical)
export(dist_cauchy)
export(dist_chisq)
export(dist_degenerate)
export(dist_dirichlet)
export(dist_exponential)
export(dist_f)
export(dist_gamma)
export(dist_geometric)
export(dist_gev)
export(dist_gh)
export(dist_gk)
export(dist_gpd)
export(dist_gumbel)
export(dist_horseshoe)
export(dist_hypergeometric)
export(dist_inflated)
export(dist_inverse_exponential)
export(dist_inverse_gamma)
export(dist_inverse_gaussian)
export(dist_laplace)
export(dist_logarithmic)
export(dist_logistic)
export(dist_lognormal)
export(dist_missing)
export(dist_mixture)
export(dist_multinomial)
export(dist_multivariate_normal)
export(dist_multivariate_t)
export(dist_negative_binomial)
export(dist_normal)
export(dist_pareto)
export(dist_percentile)
export(dist_poisson)
export(dist_poisson_inverse_gaussian)
export(dist_sample)
export(dist_student_t)
export(dist_studentized_range)
export(dist_transformed)
export(dist_truncated)
export(dist_uniform)
export(dist_weibull)
export(dist_wrap)
export(generate)
export(has_symmetry)
export(hdr)
export(hilo)
export(is_distribution)
export(is_hdr)
export(is_hilo)
export(kurtosis)
export(likelihood)
export(log_likelihood)
export(new_dist)
export(new_hdr)
export(new_hilo)
export(parameters)
export(skewness)
export(support)
export(variance)
import(rlang)
import(vctrs)
importFrom(generics,generate)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecated)
importFrom(pillar,get_max_extent)
importFrom(pillar,new_ornament)
importFrom(pillar,new_pillar_shaft)
importFrom(pillar,pillar_shaft)
importFrom(stats,density)
importFrom(stats,family)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(utils,.DollarNames)
