% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hilo.R
\name{is_hilo}
\alias{is_hilo}
\title{Is the object a hilo}
\usage{
is_hilo(x)
}
\arguments{
\item{x}{An object.}
}
\description{
Is the object a hilo
}
